'' ILI9325 driver using pins P0-P11 for the display and P16-P23 for the touchscreen
'' James Moxham 2011

CON


  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1                                           ' -1 ifnot installed.
  _SD_CD = -1                                           ' -1 ifnot installed.

  _RTC_DAT = -1                                         ' -1 ifnot installed. For kye's sd driver
  _RTC_CLK = -1                                         ' -1 ifnot installed.

  _TV_Enabled = true                                    ' TV display on
  _TV_Pin = 16                                          ' group of 3 pins starting at pin 16 = 16,17,18

  Touch_Clock = 21              ' 23 old board                          ' Touch screen pins (shared with VGA and one, penirq, shared with TV)
  Touch_ChipSelect = 23         ' 22 old board
  Touch_DataIn = 22             ' 21 old board
  'Touch_Busy = 20              ' 20 old board
  Touch_DataOut = 20            ' 19 old board
  'Touch_Penirq = 18             ' 18 old board

  FontTable = 170000
  
OBJ
 delay:         "timing"                                ' thanks to parallax
 fat:           "SD3.01_FATEngine.spin"                 ' thanks to Kye
 text:          "tv_text"                               ' thanks to Chip
 str:           "ASCII0_STREngine.spin"                 ' thanks to Kye 
 spi:           "SPI_ASM"                               ' thanks to Beau
 ILI9325:       "ILI9325_Dracblade"                     ' J Moxham

VAR
   byte LineOfText1[20] ' general purpose string buffer
   byte LineOfText2[20] ' general purpose string buffer
   byte LineOfText3[20] ' general purpose string buffer
   byte FontSize                ' size of the current loaded font (pixels = fontsize *.75)
   word curx                    ' current cursor x position
   word cury                    ' current cursor y position   
  
PUB Main
    text.start(_TV_Pin)                                 ' start the TV driver
    text.clearscreen                                    ' clear the screen
    ILI9325.start_ram                                   ' start the external ram driver in a cog
    ILI9325.start_ILI9325                               ' start the ILI9325 display
     PrintStringCR(string("Testing for SD card"))       ' helpful message if card is out, better than just a blank screen  
    'Touchscreen                                        ' test the touchscreen
    ifnot( fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))   
      reboot
    fat.mountPartition(0)                               ' mount the sd card
    TouchStart
    'OUTA[Touch_ChipSelect] := 0                         ' enable the touch screen
    'SPI.start(3,0)                                      ' delay,state for the touchscreen
    
    'createicon      ' only do once per icon, saves a .icn file to the sd card, takes 8 seconds. Or use vb.net program
    Clearscreen
    ' Font experiments
    'str.copy(string("mssans12"),@lineoftext1)           ' font name (no extension), adds .raw and .fnt to this
    'ILILoadfont(172000)                            ' font experiments
    'ILISetcursor(0,0)                              ' set cursor to this location
    'ILIprintstringcrlf(string("Microsoft Sans Serif 12 loaded"))          ' prints at curx,cury
    'ILIdecimal(20)                                     ' test debugging on the ILI screen.
    'ILIcrlf 
    'ILIHexbyte(100)                                    ' need a global x,y variable and work out y from the height of that font so can do a CRLF

{
    str.copy(string("Cour16"),@lineoftext1)           ' font name (no extension), adds .raw and .fnt to this
    Loadfont(172000)                    ' font experiments
    ILIstr(string("Courier 16"),20,50)
    
    str.copy(string("Arial30"),@lineoftext1)           ' font name (no extension), adds .raw and .fnt to this
    Loadfont(172000)                    ' font experiments
    ILIstr(string("Arial 30"),20,70)

    str.copy(string("Dig60"),@lineoftext1)           ' font name (no extension), adds .raw and .fnt to this
    Loadfont(172000)                    ' font experiments
    ILIstr(string("DIG 60"),20,110)
    
    repeat
    'Clearscreen

    
}

    DrawBackgroundAndIcons 
    PrintStringCR(string("touch the screen"))
    Touchscreen

PUB DrawBackgroundAndIcons
    str.copy(string("beach.raw"),@lineoftext1)
    FullScreenRaw                                       ' load picture 240 wide, 320 high
    DisplayIcons

PUB PhotoAlbum | picturenumber
   picturenumber := 1
   repeat until picturenumber > 4
     if picturenumber == 1
        str.copy(string("cat.raw"),@lineoftext1)    
     if picturenumber == 2
        str.copy(string("kluger.raw"),@lineoftext1)
     if picturenumber == 3
        str.copy(string("elle.raw"),@lineoftext1)    
     if picturenumber == 4
        str.copy(string("balloon.raw"),@lineoftext1)
     FullScreenRaw                                      ' draw picture name loaded into lineoftext1
     result := Touchswish
     if result > 0
       picturenumber --
     else
       picturenumber ++


PUB ILILoadFont(ILIFontData) | ramaddress,vari,ascii,x,y,width,height,nbytes,v,vara,varb,filesize
  ' use http://www.angelcode.com/products/bmfont/  to create a font
  ' in Options/Font Settings check the Font Smoothing box. Select the size. Select italic or bold if required.
  ' Load in a font and then highlight characters from Space to ~   (Ascii 32 to 126). 
  ' Options/Export Options - leave as defaults except output as a binary file (you can output as a text file to see what it looks like)
  ' Options/Save Bitmap Font as - and save it. Saves two files, a .fnt which is the descriptor file, and a .tga file which is a 256x256 picture
  ' Open Paint Shop Pro and open the .tga file. Maybe do a Colors/Negative on this so black on white. Save as a .raw file (one byte per pixel, gray scale)
  ' copy Myfont.fnt and Myfont.raw to the sd card.
  ' Read in the .raw file. 64k (256x256) and store temporarily in ram
  ' read in the .fnt file and create a new list. For sending to the ILI9325 need data in a linear format
  ' ie for a font descripting of A need to send the size of the display to display in, then send all the bytes
  ' one after the other.
  ' initially do the .raw to .ili format on the fly and see what the speed is.
  ' Need to store in an jump table array. Do all the characters in order 32 to 126.
  ' For each character; x,y,width,height,xoffset, yoffset,memory location. 6 bytes per character plus 1 long = 10 bytes
  ' So the array has 10 x 95 = 950 bytes. Memory location is the location in ram where that character is stored.
  ' decoding the data format of the binary file, (in hex)
  ' addit - the start of the data varies with the name of the font and the filename, so use hexedit to determine this
  ' byte 53 = ascii value                               (n+0)
  ' byte 57 = x                                         (n+4)
  ' byte 59 = y                                         (n+6)
  ' byte 5B = width                                     (n+8)
  ' byte 5D = height                                    (n+A)
  ' byte 5F = xoffset                                   (n+C)
  ' byte 61 = yoffset                                   (n+E)
  ' and then start again at byte 67.                    (n+14h)  = n+20
  ' so there are 5 longs per character
  ' modify this so store x,y,width,height,xoffset,yoffset,null,null so 2 longs per character, makes it easier to calculate offset jumps
  ' first, load in the .raw file. Can reuse this later (@ address 0 for the moment)
  ' Note the name should be 8 characters or less. For this example, par16.fnt and par16.raw (16 point parallax font) 
    'PrintStringCR(string("Font load par16.raw"))                 ' message on the TV screen
    'fat.openfile(string("par16.raw"),"R")            ' open a raw font image - always 64k
    str.stringcopy(@lineoftext3,@lineoftext1)'          make a copy
    str.copy(string(".raw"),@lineoftext2)
    str.stringConcatenate(@LineofText3,@LineOfText2)
    'printstringcr(@lineoftext3)
    'fat.openfile(string("Cour16.raw"),"R")            ' open a raw font image - always 64k
    fat.openfile(@lineoftext3,"R")                      ' open a raw font image - always 64k    
    ramaddress := 0
    repeat 256                                          ' for some reason if do in 512 byte blocks gives errors
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card
      ILI9325.docmd("A",@sdbuffer,ramaddress,256)       ' move data from hub to external ram 512 bytes at a time    
      ramaddress += 256                                 ' add 256
    fat.closefile   
    'PrintStringCR(string("Load .fnt descriptor file"))                 ' message on the TV screen
    ' the ILI display prefers a continuous stream of bytes and the data in the 64k block is in rows
    ' so need to convert to 2 byte ILI format and at the same time, convert to continous data
    ' lookup table better to read in and then modify one long as the jump table.
    ' as there is not much difference in the two formats
    'fat.openfile(string("par16.fnt"),"R")               ' open the decoder file
    str.copy(string(".fnt"),@lineoftext2)
    str.stringConcatenate(@LineofText1,@LineOfText2)
    'printstringcr(@lineoftext1)  
    'fat.openfile(string("Cour16.fnt"),"R")               ' open the decoder file
    fat.openfile(@lineoftext1,"R")                      ' open the decoder file  
    filesize := 2000 ' read more than the file size (usually 1987 or so) as don't know how big it is yet. 
    'printdecimal(filesize)     
    ramaddress := FontTable
    repeat filesize                                     ' size of the font file ignoring kerning block
      vari := fat.readbyte
      ILI9325.WriteRamByte(vari,ramaddress)                ' store one byte to ram
      ramaddress ++
    fat.closefile
    'PrintStringCR(string("Close"))
    'printdecimal(vari) ' last byte is 15 - debugging
    ' lookup table - ascii value
    ' 0 = mem location = 4 bytes
    ' 4 = x, 6=y,8=width,10=height,12=xoffset,14=yoffset 
     ramaddress := ILIFontData                             ' font stored here
     fontsize := ILI9325.ReadRamByte(FontTable+9)          ' read the size of the font (global variable) pixels = fontsize * .75
     repeat ascii from 32 to 126
       'printdecimal(ascii)
       vari := ((ascii - 32) * 20) + FindFontData             '  start of the data
       x := ILI9325.ReadRamByte(FontTable+vari+4)             ' read x
       y := ILI9325.ReadRamByte(FontTable+vari+6)             ' read y
       width := ILI9325.ReadRamByte(FontTable+vari+8)         ' read width
       height := ILI9325.ReadRamByte(FontTable+vari+10)       ' read height
       ILI9325.WriteRamLong(ramaddress,FontTable+vari)              ' store ILI format file location lookup table
       ' now we have the x,y location can read the bytes. Do one row at a time.
       ' make RGB all the same, then use the E command     
       repeat varb from 0 to (height - 1)                     ' get the block of data
         repeat vara from 0 to (width - 1)
            v := ILI9325.ReadRamByte(((y+varb) << 8)+x+vara)  ' row * 256+ column
            !v                                          ' invert color so black on white
            'printdecimal(v)
            sdbuffer[0] := v                            ' if all the same RGB = black on white. Set one as 255 for color on white
            sdbuffer[1] := v
            sdbuffer[2] := v
            ILI9325.docmd("E",@sdbuffer,@buffer2,1)         ' convert to 2 byte .ili format, len is number of pixels
            ILI9325.WriteRamByte(buffer2[0],ramaddress)
            ramaddress ++
            ILI9325.WriteRamByte(buffer2[1],ramaddress)
            ramaddress ++

PUB ILISetCursor(x,y)            
    curx := x
    cury := y

PUB ILIChar(ascii,x,y) | i,width,height,ramaddress,xoffset,yoffset,xadvance             
     ' font is now stored in ILI format at address ILIFontData with a lookup table at FontTable
     ' try printing a character
     'ascii := 65
     'printdecimal(ascii)
     i := ((ascii - 32) * 20) + FindFontData            ' find start of the font data see hexedit location of lookup table
     width := ILI9325.ReadRamByte(FontTable+i+8)
     height := ILI9325.ReadRamByte(FontTable+i+10)
     xoffset := ILI9325.ReadRamByte(FontTable+i+12)
     yoffset := ILI9325.ReadRamByte(FontTable+i+14)
     xadvance := ILI9325.ReadRamByte(FontTable+i+16) ' how much to advance to the next character 
     ramaddress := ILI9325.ReadRamLong(FontTable+i)
     ILI9325.draw(x+xoffset,y+yoffset,x+width-1+xoffset,y+height-1+yoffset)
     ILI9325.docmd("C",0,ramaddress,(width*height) << 1)              ' move bytes from ram out to the display
     return x + xadvance                                   ' so calling routine knows where to put next char

PUB ILIPrintstring(stringptr)                 'print at curx,cury
'' Print a zero-terminated string
  repeat strsize(stringptr)
    curx := ILIChar(byte[stringptr++],curx,cury)

PUB ILIPrintstringcrlf(stringptr) ' print string and crlf
    ILIprintstring(stringptr)
    ILIcrlf   

PUB ILIDecimal(number)
      str.str(@lineoftext1,number) ' decimal to string with no leading zeros
      ILIPrintstring(@lineoftext1)

      'str.copy(str.integertodecimal(number,10),@lineoftext1) ' or add lots of leading zeros. Your choice
      'ILIstr(@lineoftext1) 

PUB ILIHexByte(number)      ' print a byte in hex (can do longer if you want, just change the number of chars)
     str.copy(str.integertohexadecimal(number,2),@lineoftext1)
     ILIPrintstring(@lineoftext1)

PUB ILIcrlf
    curx := 0
    cury := cury + (fontsize * 30) / 40                 ' pixel height happens to be fontsize * .75           
                     

PUB FindFontData | position ' see instructions in Start/Programs/AngelCode/Bitmap Font Generator/Documentation
    ' Each block starts with a one byte block type identifier, followed by a 4 byte integer that gives the size of the block, not including the block type identifier and the size value.
    ' so we start with block 1 which is at position 4
    ' read the  block size which is position 5 and add 3 bytes. Use a hex editor to understand all this
    position := ILI9325.ReadRamByte(FontTable+5)
    position += 10                                       ' 4 bytes BMF3, then block 1, then 4 bytes for size
    position := position + ILI9325.ReadRamByte(FontTable+position) + 5 ' add size of block 2
    position := position + ILI9325.ReadRamByte(FontTable+position) + 5 ' add size of block 3
                                         ' this is the size of the font data block but ignore as it is always fixed for ascii 32 to 126 inclusive                                       
    position += 4                                           ' should point to beginning of data   
    return position


PUB ReadBackground | ramaddress             ' block move data from sd card to memory, much faster than one byte at a time
    PrintStringCR(string("Block load"))                 ' message on the TV screen
    fat.openfile(string("icons320.ili"),"R")            ' open an image (.ili extension)
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card
      ILI9325.docmd("A",@sdbuffer,ramaddress,256)       ' move data from hub to external ram 256 bytes at a time    
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile   
    PrintStringCR(string("Closing file"))
    RamToDisplay


PUB FullScreenRaw | ramaddress                         ' convert 3 byte .raw file to 2 byte .ili file
    fat.openfile(@lineoftext1,"R")                ' open an image
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' move bytes to external ram
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile
    RamToDisplay                              ' move bytes from ram out to display

PUB DrawKeyboard | ramaddress
    ' reload the font as gets overwritten by icons
    str.copy(string("mssans12"),@lineoftext1)           ' font name (no extension), adds .raw and .fnt to this
    ILILoadfont(172000)                            ' font experiments
    fat.openfile(string("keyboard.raw"),"R")                ' open the keyboard picture   
    ramaddress := 0
    repeat 360
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' move 256 bytes to external ram (must be 256)
      ramaddress += 256                                 ' add 256
    fat.closefile
    ILI9325.draw(0,128,239,319)                           ' 240x192 for the keyboard
    ILI9325.docmd("C",0,0,92160)                       ' move bytes from ram out to the display

PUB Touchkeyboard  | yval, xval,address,ascii,index,capslock, backspace
    ' from the top of the keyboard picture - add 128 for screen pixel 0-319
    ' y=33-61 is the numbers ( 161 - 189 which is 50% to 59%) 
    ' y = 64-93 is qwerty    (192 to 221 which is 60 to 69%)
    ' y = 96-124 is asdf      (224 to 252 which is 70-79%)
    ' y= 128-157 is zxcv      (256 to 285 which is 80 to 89%)
    ' y=160-189 is space
    capslock := 0 ' default to off
    backspace := 0 'default to off
    ILISetcursor(10,137)                              ' set cursor to this location    
    repeat
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)               ' a valid value
        printdecimal(xval)                              ' display the numbers on TV
        printdecimal(yval)
        crlf
        if (yval => 50) and (yval =< 59)                ' number row 0-100%
          address := xval / 10                              ' 10 values in row = 0 to 9
          'ILIdecimal(address)                                     ' test debugging on the ILI screen.

        if (yval => 60) and (yval =< 69)
          address := 10 + (xval / 10)                   ' address = 10 to 19    QWERTYUIOP

        if (yval => 70) and (yval =< 79)
          address := 20 + ((xval - 5) / 10)             ' address = 20-28 ASDFGHJKL shift left slightly as this row is offset half a key

        if (yval => 80) and (yval =< 89)                ' third row
          if (xval < 15)
            !capslock                                   ' swap 0,FFFFFFFF capslock
            address := 255                              ' so no character printed
            printstringcr(string("Caps lock"))
          if (xval =>15) and (xval =< 85)
            address := 29 + ((xval - 15) / 10)          ' ZXCVBNM
          if xval > 85
            backspace := 255                             ' backspace - need to be storing in a string and delete the last character and reprint the string                             

        if yval => 90                                   ' fourth row
           if xval <25
           ' not sure what to do with this 123 button, maybe all the !@#$%^ characters above the numbers
           if (xval => 25) and (xval =< 75)
             address := 36                              ' space  button
           if xval > 75                                 ' enter button
           ' exit here and return a string
           
        if address < 255                                ' not a backspace or an enter or a capslock
          ascii := lookupz(address: "1","2","3","4","5","6","7","8","9","0","Q","W","E","R","T","Y","U","I","O","P","A","S","D","F","G","H","J","K","L","Z","X","C","V","B","N","M"," ")
          if (capslock <> 0) and (ascii => 65) and (ascii =< 90) 
            ascii += 32                                 ' lower case
          curx := ILIChar(ascii,curx,cury)
             
      delay.pause1ms(250)' only display once so delay half a second to "debounce"   
            
    
PUB CreateIcon | iconaddress,maskaddress,i,j
  ' this creates a .icn file which is 4 bytes per pixel, red, green, blue and alpha. Do once when setting up icons
  ' so one long per pixel. All values are 0-255
  ' takes 8 seconds per icon so don't do this every load
  ' to create an icon using paint shop pro or similar
  ' in the catalina IDE this has been automated to a one click process for any .png file
  '1) Open a .png icon file which has a transparancy layer
  '2) Copy and paste the picture to a new picture and save as a .raw RRRRRRRRGGGGGGGGBBBBBBBB file
  '3) View the mask (in PSP there is a Mask on the top menu bar, then Edit).
  '4) Masks/Save the file, and rename the extension as .bmp.
  '5) Close down all open files in PSP, then File/Open and open the .bmp file above. This should be 256 color grayscale, ie 1 byte per pixel.
  '6) Save that file as a .raw file, eg mask.raw
  '7) Copy the mask.raw file to the sd card This .raw file has 1 byte per pixel (note different to the picture which is 3 bytes per pixel)    

  ' in a family of icons the mask.raw file is going to be the same for all of them
  ' so on the sd card one might have two files - clock.raw and mask.raw
  ' this function combines these together into an .icn file. It is in spin which is slower but this is only ever run once
  PrintStringCR(string("myfile.raw and mask.raw to myfile.icn"))   
  fat.openfile(string("clock.raw"),"R")                 ' open an icon file which is 59x60 bytes x3 = 10620
  iconaddress := 0 
  repeat 10620                                          ' 
    i := fat.readbyte
    ILI9325.docmd("W",@i,iconaddress,1)                  ' write n to ramaddress
    iconaddress +=1                                     ' increment the counter
  fat.closefile
  fat.openfile(string("mask.raw"),"R")                 ' open an icon file which is 59x60 bytes x3 = 10620
  maskaddress := 16384
  repeat 3540                                           ' one byte per pixel in the mask
    i := fat.readbyte
    ILI9325.docmd("W",@i,maskaddress,1)                  ' write n to ramaddress
    maskaddress += 1                                    ' increment the counter
  fat.closefile
  iconaddress := 0                                      ' reset counters
  maskaddress := 16384
                                                        ' see kyedos for using "stringpointer" instead of text 
  result := \fat.newFile(string("clock.icn"))           ' All aborts traps must have "variable := \function" setup.
  if(fat.partitionError == fat#Entry_Already_Exist)     ' file already exists
     result := \fat.deleteEntry(string("clock.icn"))    ' delete the old file
     result := \fat.newfile(string("clock.icn"))        ' recreate the file
  fat.openfile(string("clock.icn"),"W")
  repeat 3540
    j := 0                                              ' clear j
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram into i = the red byte
    iconaddress += 1
    j := j | i                                          ' put the red byte into lsb place
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram
    iconaddress += 1
    j := j | (i << 8)                                   ' put the green byte into place
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram 
    iconaddress += 1
    j := j | (i << 16)                                  ' put the blue byte into place
    ILI9325.docmd("R",@i,maskaddress,1)                 ' read from ram into i the alpha byte
    maskaddress += 1
    j := j | (i << 24)                                  ' put the alpha byte into msb place - hex editor = RGBA
    fat.writelong(j)                                    ' store to SD card file
  fat.closefile
  PrintStringCR(string("clock.icn created"))                                                                           

PUB RamToDisplay ' dumps preloaded ram to display starting address 0
    'Clearscreen                                        ' leave out clearscreen
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in
    ILI9325.docmd("C",0,0,153600)                       ' move bytes from ram out to the display

PUB Movie                                              ' maybe need to speed up the draw routine as this is in spin
    clearscreen
    fat.openfile(string("taz.ilm"),"R")             ' also can skip alt frames, and maybe make the sd buffer bigger
    repeat 338 ' number of frames
      ILI9325.draw(60,115,179,204)                           ' this can be sped up as this is in spin
      repeat 45                                         ' 120*90*2 / 480
        fat.readdata(@sdbuffer,480)                     ' read data into hub
        ILI9325.docmd("D",@sdbuffer,0,480)              ' send data out to the display (could possibly do some in parallel, though the ILI display may be faster than the sd card)
    fat.closefile
PUB Clearscreen | i
    i := 0
    repeat 512
      sdbuffer[i] := 255                         ' array all to zero = black, 255 = white, other colors need to work out the RRRRRGGG GGGBBBBB values
      i := i + 1
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in   
    repeat 300
      ILI9325.docmd("D",@sdbuffer,0,512)                       ' move bytes from ram out to the display


PUB TransparentIcon(x,y) | sizex,sizey,backgroundaddress,ramaddress ' draw icon name in LineOfText1
    x := x * 60 ' pass x as 0,1,2,3 from touchscreen convert to a pixel coordinate
    y := y * 60 ' pass y as 0,1,2,3,4 from touchscreen
    sizex := 59
    sizey := 60
    ILI9325.draw(x,y,x+sizex-1,y+sizey-1)                           ' set up the area of the screen to draw in
     ' background is stored in memory 0 to 153599
     ' background behind the icon temp store =64*64*4 = 16384 bytes 153600 to 169984 so contiguous bytes
    backgroundaddress := (((y*240)+x) << 1)              ' calculate the correct location in background ram
    ramaddress := 153600
    repeat 30
      ILI9325.docmd("R",@sdbuffer,backgroundaddress,(sizex << 2)) ' get 59*2 *2 bytes from background. Instead of 60 lines with everything multiplied by 2, do 30 lines and multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' save to a temp buffer 
      backgroundaddress += (240 << 2)                          ' next row = 240 pixels *2  *2
      ramaddress += (sizex << 2)                        ' 59*2  *2
    ramaddress := 169984 ' location to store icon
    'PrintStringCR(string("Load"))            ' message on the TV screen
    fat.openfile(@lineoftext1,"R")              ' open an icon image (.icn extension = RGBA)
    repeat sizey
      fat.readdata(@sdbuffer,(sizex << 2))                 ' sizex pixels * 4 bytes per pixel so multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' now move data from hub to external ram
      ramaddress += (sizex << 2)                        ' 4 bytes per pixel so multiply by 4   
    fat.closefile   
    'PrintStringCR(string("Closing file"))
    ILI9325.docmd("F",153600,169984,(sizex*sizey))  ' background,icon,number of pixels


PUB DisplayIcons
    'str.copy(string("clock.icn"),@lineoftext1) ' copy fixed value into a string
    'TransparentIcon(1,1) ' display icon in lineoftext1
    'str.copy(string("cpm.icn"),@lineoftext1) ' copy fixed value into a string  
    'TransparentIcon(2,1) ' display icon in lineoftext1
    str.copy(string("photos.icn"),@lineoftext1) ' copy fixed value into a string
    TransparentIcon(1,1) ' display icon in lineoftext1
    str.copy(string("videos.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(2,4) ' display icon in lineoftext1
    str.copy(string("key.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(3,0) ' display icon in lineoftext1  

PUB TouchStart
  OUTA[Touch_ChipSelect] := 0                           ' enable the touch screen
  SPI.start(3,0)                                        ' delay,state

PUB TouchValue | xval,yval ' returns % values 0-100% (or 255 for no touch)
'  Touch_Clock = 21                                       ' clock
'  Touch_ChipSelect = 23                                  ' chip select needs to be low to enable the touchscreen
'  Touch_DataIn = 22                                      ' data into the chip = an output from the prop
'  Touch_Busy = 20                                        ' busy ? not needed - just returns /cs
'  Touch_DataOut = 20                                     ' data out of the chip = an input to the prop
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1101_0000)  ' reads x from  500 to 3700  (off < 500 )
      xval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1001_0000)  ' reads y from 400 to 3800 (off > 3800 ) 
      yval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      if xval <500                                      ' if less than 500 then not touched, change to 255
        xval := 255                                      ' no touch is 255
      else ' convert 3800-440 to 0-100%
        xval -=  440                                    ' scale 0-3360
        xval /=  32                                     ' scale 0-100
        xval <#= 100                                    ' max 100
        xval #>=0                                       ' min 0
        xval := 100 - xval                              ' so left edge on left
        'printdecimal(xval) 
      if yval > 3800                                    ' if >3800 then not touched, change to 255
        yval := 255                                     ' no touch is 255
      else  ' 3600 is top, 420 is bottom
        yval -= 420                                     ' scale 0-3180
        yval /= 32                                      ' scale 0-100
        yval <#= 100                                    ' max 100
        yval #>=0                                       ' min 0
        yval := 100-yval                                ' so 0 is top left
        'printdecimal(yval)
      result := (yval <<8) | xval                       ' return 00000000_00000000_YYYYYYYY_XXXXXXXX

PUB TouchSwish | xval,oldxval,difference,average ' returns result = +ve or -ve depending on swish direction     ' did finger move left or right?
      oldxval := 255' for startup
      average := 0
      repeat until (average > 30) or (average < -30)
        ' possibly add a decay function here for average
        delay.pause1ms(50)
        xval := TouchValue & 255                        ' decode xval 0-100%
        if xval == 255                                  ' not touched
          oldxval := 255                                ' reset oldxval as well
        else
          if oldxval <>255                                ' discard the first reading as the screen is touched
            difference := xval - oldxval
            'printdecimal(xval)
            'printdecimal(difference)
            average := average + difference             ' 
            'printdecimal(average)
            'crlf
          oldxval := xval                                 ' store for next time
      result := average                                 ' return negative or positive value     


PRI TouchScreen  | yval, xval,x,y
    repeat
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)
        x := xval / 25                                  ' scale 0,1,2,3
        y := yval / 20                                  ' 0,1,2,3,4
        printdecimal(xval)                              ' display the numbers on TV
        printdecimal(yval)
        crlf
        if (x == 0) and (y == 0)
            str.copy(string("twitter.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 0)
            str.copy(string("calc.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 0)  
            str.copy(string("calendar.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 0)
            DrawKeyboard                 ' popup keyboard
            Touchkeyboard
            
        if (x == 0) and (y == 1)  
            str.copy(string("cydia.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 1)
            'str.copy(string("facebook.icn"),@lineoftext1) ' copy fixed value into a string
            'TransparentIcon(x,y) ' display icon in lineoftext1
            PhotoAlbum
            DrawBackgroundAndIcons   
        if (x == 2) and (y == 1)
            str.copy(string("contacts.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 1)
            str.copy(string("cpm.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 2)  
            str.copy(string("ifile.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 2)
            str.copy(string("itunes.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 2)  
            str.copy(string("phone.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 2)
            str.copy(string("mail.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 3)  
            str.copy(string("maps.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 3)
            str.copy(string("messages.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 3)
            str.copy(string("music.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 3)
            str.copy(string("notes.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 4)  
            str.copy(string("macri.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 4)
            str.copy(string("safari.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 4)  
            'str.copy(string("settings.icn"),@lineoftext1) ' copy fixed value into a string
            'TransparentIcon(x,y) ' display icon in lineoftext1
            Movie ' play movie if hit the movie icon
            DrawBackgroundAndIcons
        if (x == 3) and (y == 4)
            str.copy(string("stocks.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1

        delay.pause1ms(250)' only display once so delay half a second to "debounce"

 
PUB PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PUB PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    if (_TV_Enabled)
      text.str(pstring)
'    if (_VGA_Enabled)  
'      vt100.str(pstring)                                 ' send to vga screen
'    if (_Serial_Enabled)   
'    sio.str(0,pstring)                                 ' send to com port
'    if (_lcd_Enabled)
'       lcd.str(pstring)                                   ' send to 20x4 lCD display

PUB PrintChar(c) ' sends c to vga and serial port
    if (_TV_Enabled)
      text.out(c)
'    if (_VGA_Enabled)
'      vt100.out(c)                                        ' send to vga
'    if (_Serial_Enabled)
'      sio.tx(0,c)                                         ' send to com port
'    if (_lcd_Enabled)
'      LCD.out(c)                                          ' send to LCD display      
PUB CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PUB PrintDecimal(number)                                ' print the decimal value, useful for debuggin
     PrintString(str.integerToDecimal(number, 10))

DAT
sdbuffer        byte    $0[512]  ' 512 byte buffer for sd card interface
buffer2         byte    $0[512]  ' 512 general purpose hub buffer